import random
from dataclasses import asdict, dataclass, field, fields
from typing import List

from mimesis import Address, Generic, Person
from mimesis.providers.base import BaseProvider
from rich.progress import track
from rich import markup
from azdummy import settings
from azdummy.styles import AZURE_BOLD

person = Person("en")
address = Address(settings.AZD_LOCALE)
generic = Generic(settings.AZD_LOCALE)


@dataclass
class AzUser:
    """Dataclass for creating an AzUser object"""

    domain: str = field(repr=False, hash=False)
    userPrincipalName: str = field(init=False)
    name: str = field(init=False)
    passwordProfile: str = field(init=False)
    blockSignIn: bool = False
    givenName: str = field(init=False)
    surname: str = field(init=False)
    jobTitle: str = "Employee"
    department: str = field(init=False)
    usageLocation: str = field(init=False)
    streetAddress: str = field(init=False)
    state: str = field(init=False)
    country: str = field(init=False)
    physicalDeliveryOfficeName: str = None
    city: str = field(init=False)
    postalCode: str = field(init=False)
    telephoneNumber: str = field(init=False)
    mobile: str = field(init=False)

    def __post_init__(self):
        self.name = person.full_name()
        while "'" in self.name:
            self.name = person.full_name()

        self.givenName, self.surname = self.name.split()
        self.userPrincipalName = f"{self.givenName}.{self.surname}@{self.domain}"
        self.passwordProfile = markup.escape(person.password(length=20))
        self.department = random.choice(settings.AZD_GROUP_NAMES)
        self.streetAddress = f"{address.street_number()} {address.street_name()} {address.street_suffix()}"
        self.state = address.state(True)
        self.city = address.city()
        self.postalCode = address.postal_code()
        self.usageLocation = address.country()
        self.country = address.country()
        self.telephoneNumber = person.telephone("###-###-####")
        self.mobile = person.telephone("###-###-####")
        self.blockSignIn = "Yes" if self.blockSignIn else "No"

    def to_list(self) -> list:
        """Returns the AzUser object fields as a list"""
        return [
            v
            for k, v in asdict(self).items()
            if k
            not in [a.name for a in filter(lambda x: x.repr == False, fields(self))]
        ]


class AzureADProvider(BaseProvider):
    class Meta:
        name = "AzureADProvider"

    @staticmethod
    def AzUsers(domain: str, number: int, block_login: bool) -> List[AzUser]:
        """Returns a list of AzUsers"""

        users = []
        existingNames = set()
        for _ in track(range(number), description=f"[{AZURE_BOLD}]Generating users..."):
            while True:
                user = AzUser(domain, blockSignIn=block_login)
                if user.name not in existingNames:
                    users.append(user)
                    existingNames.add(user.name)
                    break
        return users


generic.add_provider(AzureADProvider)



import shutil
from pathlib import Path

import typer
from dotenv import load_dotenv
from rich import pretty, traceback
from rich.console import Console
from rich.prompt import Confirm

from .settings import AzDummySettings

APP_NAME = "azdummy"

state = {"verbose": False, "timeStart": None, "resetConfig": True}

console = Console()
traceback.install(show_locals=True)
pretty.install()

app_dir = typer.get_app_dir(APP_NAME)
config_path = Path(app_dir) / "settings.env"


def reset_config():
    """Reset the configuration to default"""
    try:
        Path(app_dir).mkdir(parents=True, exist_ok=True)
        shutil.copy(Path(__file__).parent / "settings.env", config_path)
    except:
        console.print_exception()


if not config_path.exists():
    if Confirm.ask(
        "\n:grimacing face: :thumbs_down_dark_skin_tone: Looks like your first time running azdummy. Would you like initialize a new config?"
    ):
        reset_config()

load_dotenv(config_path, verbose=True)
settings = AzDummySettings(config_path)



from enum import Enum

from pydantic import BaseSettings, SecretBytes, validator


class LocaleEnum(str, Enum):
    CZECH = "cs"
    DANISH = "da"
    GERMAN = "de"
    AUSTRIAN_GERMAN = "de-at"
    SWISS_GERMAN = "de-ch"
    GREEK = "el"
    ENGLISH = "en"
    AUSTRALIAN_ENGLISH = "en-au"
    CANADIAN_ENGLISH = "en-ca"
    BRITISH_ENGLISH = "en-gb"
    SPANISH = "es"
    MEXICAN_SPANISH = "es-mx"
    ESTONIAN = "et"
    FARSI = "fa"
    FINNISH = "fi"
    FRENCH = "fr"
    HUNGARIAN = "hu"
    ICELANDIC = "is"
    ITALIAN = "it"
    JAPANESE = "ja"
    KAZAKH = "kk"
    KOREAN = "ko"
    DUTCH = "nl"
    BELGIUM_DUTCH = "nl-be"
    NORWEGIAN = "no"
    POLISH = "pl"
    PORTUGUESE = "pt"
    BRAZILIAN_PORTUGUESE = "pt-br"
    RUSSIAN = "ru"
    SLOVAK = "sk"
    SWEDISH = "sv"
    TURKISH = "tr"
    UKRAINIAN = "uk"
    CHINESE = "zh"


class AzDummySettings(BaseSettings):
    """Settings for AzDummy"""

    AZD_AUTH_MODE: str
    AZD_TENANT_FQDN: str
    AZD_LOCALE: LocaleEnum
    AZD_CLIENT_ID: str
    AZD_CLIENT_SECRET: SecretBytes
    AZD_NUM_USERS: int
    AZD_NUM_GLOBAL_ADMIN: int
    AZD_BLOCK_LOGIN: bool
    AZD_GROUP_NAMES: list

    @validator("AZD_NUM_USERS")
    def max_limit(cls, v):
        if v > 50000:
            raise ValueError("AZD_NUM_USERS needs to be 50,000 or less")
        return v

    @validator("AZD_NUM_GLOBAL_ADMIN")
    def max_global_limit(cls, v, values):
        if v > 50000:
            raise ValueError("AZD_NUM_GLOBAL_ADMIN needs to be 50,000 or less")
        elif v > values["AZD_NUM_USERS"]:
            raise ValueError(
                "AZD_NUM_GLOBAL_ADMIN number set higher than AZD_NUM_USERS"
            )



import csv
from dataclasses import fields
from enum import Enum

import typer
from rich import box
from rich.columns import Columns
from rich.panel import Panel
from rich.progress import track
from rich.table import Table

from azdummy import console, settings
from azdummy.provider import generic
from azdummy.styles import *

app = typer.Typer()


class OutputFormat(str, Enum):
    CONSOLE = "console"
    CSV = "csv"


@app.command()
def users(
    tenant: str = typer.Option(settings.AZD_TENANT_FQDN, help="Tenant name"),
    count: int = typer.Option(
        settings.AZD_NUM_USERS,
        metavar=None,
        max=50000,
        help="Number of users to generate. Max 50000.",
    ),
    format: OutputFormat = typer.Option(
        OutputFormat.CSV,
        show_choices=True,
        case_sensitive=False,
        help="Format for output.",
    ),
    block_login: bool = typer.Option(
        settings.AZD_BLOCK_LOGIN,
        "--block-login",
        "-b",
        help="Prevent generated users from logging in",
    ),
    output_file: typer.FileTextWrite = typer.Option(
        "output", help="File to output. Extension not required."
    ),
):
    """
    Generate fake user data.

    The output format determines which fields are generated. CSV format
    will output two CSVs: one for bulk create and one for bulk delete.

    Console output generates the users and outputs them to the console. Does
    not create the CSV files.
    """

    groups = {group: [] for group in settings.AZD_GROUP_NAMES}

    if format.value in [OutputFormat.CONSOLE, OutputFormat.CSV]:
        users = generic.AzureADProvider.AzUsers(tenant, count, block_login)

        if format.value == OutputFormat.CONSOLE:
            table = Table(
                title=" ",
                header_style=WHITE_BOLD,
                style=AZURE,
                box=box.MINIMAL_HEAVY_HEAD,
            )

            cols = [c.name for c in filter(lambda x: x.repr == True, fields(users[0]))]
            for col in cols:
                table.add_column(col, no_wrap=False)

            for user in track(users, description=f"[{AZURE_BOLD}]Generating table..."):
                values = []
                for col in cols:
                    values.append(user.__dict__.get(col))
                table.add_row(*values)
                groups[user.department].append(user.userPrincipalName)

            with console.status(
                f"[cyan]Generating output for {count} users. Please be patient for large datasets.\n",
                spinner="runner",
            ) as status:
                console.print(table)

            render_groups = [
                Panel(
                    "\n".join(members),
                    title=f"[{WHITE_BOLD}]{group}",
                    expand=True,
                    border_style=AZURE_BOLD,
                )
                for group, members in groups.items()
                if members
            ]
            console.print(Columns(render_groups))
        elif format.value == OutputFormat.CSV:

            # Create user create/delete CSV files
            user_headers = [
                "User name [userPrincipalName] *",
                "Name [displayName] *",
                "Initial password [passwordProfile] *",
                "Block sign in (Yes/No) [accountEnabled] *",
                "First name [givenName]",
                "Last name [surname]",
                "Job title [jobTitle]",
                "Department [department]",
                "Usage location [usageLocation]",
                "Street address [streetAddress]",
                "State or province [state]",
                "Country or region [country]",
                "Office [physicalDeliveryOfficeName]",
                "City [city]",
                "ZIP or postal code [postalCode]",
                "Office phone [telephoneNumber]",
                "Mobile phone [mobile]",
            ]
            with console.status(
                f"[bold green]Generating bulk upload and delete files. Please be patient for large datasets.\n",
                spinner="hamburger",
            ) as status:
                with open(
                    f"{output_file.name}_create.csv", "w", newline="", encoding="utf-8"
                ) as createfile:
                    with open(
                        f"{output_file.name}_delete.csv",
                        "w",
                        newline="",
                        encoding="utf-8",
                    ) as deletefile:
                        createwriter = csv.writer(createfile)
                        createwriter.writerow(
                            ["version:v1.0"] + [""] * len(user_headers)
                        )
                        createwriter.writerow(user_headers)

                        deletewriter = csv.writer(deletefile)
                        deletewriter.writerow(["version:v1.0"])
                        deletewriter.writerow(
                            ["User name [userPrincipalName] Required"]
                        )

                        for user in users:
                            createwriter.writerow(user.to_list())
                            deletewriter.writerow([user.userPrincipalName])

                            groups[user.department].append(user.userPrincipalName)

                    console.print(
                        Panel.fit(
                            f":white_heavy_check_mark: Generated {count} users. Output files: [white] \[{createfile.name}, {deletefile.name}]",
                            style=AZURE_BOLD,
                        )
                    )

            with console.status(
                f"[bold green]Generating group files. Please be patient for large datasets.\n",
                spinner="hamburger",
            ) as status:
                for group, members in groups.items():
                    if members:
                        with open(
                            f"{output_file.name}_{group.replace(' ', '_')}.csv",
                            "w",
                            newline="",
                            encoding="utf-8",
                        ) as groupfile:
                            groupwriter = csv.writer(groupfile)
                            groupwriter.writerow(["version:v1.0"])
                            groupwriter.writerow(
                                [
                                    "Member object ID or user principal name [memberObjectIdOrUpn] Required"
                                ]
                            )
                            groupwriter.writerows([[member] for member in members])

                group_filelist = ", ".join(
                    [
                        f"{output_file.name}_{group.replace(' ', '_')}.csv"
                        for group in groups
                    ]
                )
                console.print(
                    Panel.fit(
                        f":white_heavy_check_mark: Generated group files. Output files: \n[white] \[{group_filelist}]",
                        style=AZURE_BOLD,
                    )
                )



import typer
from click_help_colors import HelpColorsCommand, HelpColorsGroup


class CustomHelpColorsGroup(HelpColorsGroup):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.help_headers_color = "blue"
        self.help_options_color = "yellow"


class CustomHelpColorsCommand(HelpColorsCommand):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.help_headers_color = "blue"
        self.help_options_color = "yellow"


class AZDTyper(typer.Typer):
    def __init__(
        self,
        *args,
        cls=CustomHelpColorsGroup,
        context_settings={"help_option_names": ["-h", "--help"]},
        **kwargs
    ) -> None:
        super().__init__(*args, cls=cls, context_settings=context_settings, **kwargs)

    def command(
        self,
        *args,
        cls=CustomHelpColorsCommand,
        context_settings={"help_option_names": ["-h", "--help"]},
        **kwargs
    ) -> typer.Typer.command:
        return super().command(
            *args, cls=cls, context_settings=context_settings, **kwargs
        )



import time

import typer
from importlib_metadata import version
from rich.panel import Panel

from azdummy import console, state
from azdummy.commands import config, gen
from azdummy.core.typer import AZDTyper

app = AZDTyper(name="azdummy")
app.add_typer(config.app, name="config", help="Show or edit configuration data")
app.add_typer(gen.app, name="gen", help="Generate fake data for specified type")


def command_finish(*args, **kwargs):
    if state["timeStart"]:
        elapsedTime = time.time() - state["timeStart"]
        console.print(
            Panel.fit(f":stopwatch:  Execution time: [bold yellow]{elapsedTime}"),
            style="bold green",
        )


def version_callback(value: bool):
    if value:
        typer.echo(f"AzDummy: {version('azdummy')}")
        raise typer.Exit()


@app.callback(result_callback=command_finish)
def main(
    verbose: bool = typer.Option(False, "--verbose", "-v"),
    version: bool = typer.Option(
        False, "--version", callback=version_callback, is_eager=True
    ),
    timer: bool = typer.Option(
        False, "--timer", help="Print execution time after output"
    ),
):
    if verbose:
        state["verbose"] = True
    if timer:
        state["timeStart"] = time.time()
    print()


if __name__ == "__main__":
    app()



import typer
from dotenv import set_key
from pydantic import SecretBytes
from rich.panel import Panel
from rich.prompt import Confirm

from azdummy import config_path, console, reset_config, settings, state
from azdummy.core.typer import AZDTyper

app = AZDTyper()


@app.command()
def show(
    types: bool = typer.Option(
        False, "--types", help="Show expected field types for configuration"
    )
):
    """
    Show the configuration.

    By default, does not show secrets. If using --verbose, secrets will be shown in plaintext.
    """
    if types:
        console.print(Panel.fit("[bold yellow]Field Types[/bold yellow]"))
        console.print(settings.fields)

    output = {}
    for k, v in settings.dict().items():
        if isinstance(v, SecretBytes):
            if state["verbose"]:
                v = v.get_secret_value().decode()
            else:
                v = v.display()

        output[k] = v

    console.print(Panel.fit(f"[bold red]Config File:[/bold red] {config_path}"))
    console.print(output)


@app.command()
def launch():
    """
    Opens the configuration file in default editor
    """
    if not typer.launch(config_path.as_uri()):
        console.print()


@app.command(name="set")
def set_env_key(
    key: str = typer.Argument(
        ...,
        help="Key to set. Must exist in config.",
        case_sensitive=False,
        callback=lambda x: x.upper(),
    ),
    value: str = typer.Argument(..., help="Value to set for key."),
):
    """
    Set a configuration key/value pair.
    """
    if key not in settings.fields:
        fields_list = "\n".join([k for k in settings.fields])
        raise typer.BadParameter(f"{key}. Config options are: \n{fields_list}\n")

    res, _, _ = set_key(config_path, key, value)
    if res:
        console.print(
            f":thumbs_up_dark_skin_tone: Set {key} to {value}", style="bold green"
        )
    else:
        console.print(
            f":thumbs_down_dark_skin_tone: Could not set {key} to {value}",
            style="bold red",
        )


@app.command()
def reset():
    """
    Reset the configuration to default.
    """

    if Confirm.ask("\n:bomb: Are you sure you want to reset the config file?"):
        reset_config()
        console.print(
            Panel.fit(
                f":thumbs_up_dark_skin_tone: Successfully reset the config file: {config_path} :thumbs_up_dark_skin_tone:"
            ),
            style="bold green",
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['azdummy', 'azdummy.commands', 'azdummy.core']

package_data = \
{'': ['*']}

install_requires = \
['click-help-colors>=0.9,<0.10',
 'mimesis>=4.1.2,<5.0.0',
 'pydantic>=1.7.3,<2.0.0',
 'python-dotenv>=0.15.0,<0.16.0',
 'rich>=9.3.0,<10.0.0',
 'typer[all]>=0.3.2,<0.4.0']

entry_points = \
{'console_scripts': ['azdummy = azdummy.main:app']}

setup_kwargs = {
    'name': 'azdummy',
    'version': '1.1.1',
    'description': 'A Python Typer-based CLI tool to generate fake data for Azure AD.',
    'long_description': '<div align="center">\n    <img src="https://github.com/daddycocoaman/AzDummy/raw/main/docs/images/AzDummy.png" width="400px" height="400px"/>\n</div>\n\n<div align="center">\n    <img src="https://img.shields.io/pypi/v/azdummy"/>\n    <img src="https://img.shields.io/pypi/pyversions/azdummy"/>\n    <img src="https://img.shields.io/pypi/l/azdummy"/>\n    <a href="https://twitter.com/mcohmi"><img src="https://img.shields.io/twitter/follow/mcohmi.svg?style=plastic"/></a>\n</div>\n\n# AzDummy\nA Python [Typer-based](https://github.com/tiangolo/typer) CLI tool to generate fake data for Azure AD. AzDummy also uses [Rich](https://github.com/willmcgugan/rich) for some dope console output.\n\n## Installation\n\nThe recommended method of installation is with [pipx](https://github.com/pipxproject/pipx). \n\n```\npipx install azdummy\n```\n\nHowever, you can install the normal way from PyPi with `python3 -m pip install azdummy`.\n\n## Usage\n\nOn first run, user will be prompted to create a config file. Location of this config file depends on OS. **Note: There are some environment variables included that are currently not used.**\n\n- Windows: \n  - `C:\\Users\\<user>\\AppData\\Roaming\\azdummy\\settings.env`\n- Linux/Mac OS: \n  - `~/.azdummy/settings.env`\n\nCurrently used variables:\n\n- **AZD_LOCALE**: (str) Shortcode for supported locales\n- **AZD_TENANT_FQDN**: (str) One of the domains in the tenant (Usually `<domain>.onmicrosoft.com` format)\n- **AZD_NUM_USERS**: (int) Number of users to generate  \n- **AZD_BLOCK_LOGIN**: (bool) Block generated users from logging in\n- **AZD_GROUP_NAMES**: (list) List of groups to add users to\n\n**NOTE: Due to restrictions on the userPrincipalName field, all names are generated in English. However, AzDummy supports other locale-specific data generation (such as addresses).**\n\n**Supported Locales:**\n- CZECH = "cs"\n- DANISH = "da"\n- GERMAN = "de"\n- AUSTRIAN_GERMAN = "de-at"\n- SWISS_GERMAN = "de-ch"\n- GREEK = "el"\n- ENGLISH = "en"\n- AUSTRALIAN_ENGLISH = "en-au"\n- CANADIAN_ENGLISH = "en-ca"\n- BRITISH_ENGLISH = "en-gb"\n- SPANISH = "es"\n- MEXICAN_SPANISH = "es-mx"\n- ESTONIAN = "et"\n- FARSI = "fa"\n- FINNISH = "fi"\n- FRENCH = "fr"\n- HUNGARIAN = "hu"\n- ICELANDIC = "is"\n- ITALIAN = "it"\n- JAPANESE = "ja"\n- KAZAKH = "kk"\n- KOREAN = "ko"\n- DUTCH = "nl"\n- BELGIUM_DUTCH = "nl-be"\n- NORWEGIAN = "no"\n- POLISH = "pl"\n- PORTUGUESE = "pt"\n- BRAZILIAN_PORTUGUESE = "pt-br"\n- RUSSIAN = "ru"\n- SLOVAK = "sk"\n- SWEDISH = "sv"\n- TURKISH = "tr"\n- UKRAINIAN = "uk"\n- CHINESE = "zh"\n## Commands\n\nCommands are available [here](docs/commands.md). You can generally use `--help/-h` for any command or subcommand for more information. With default settings, the following command will generate 500 users for `azdummy.onmicrosoft.com` (non-existant tenant).\n\n`azdummy gen users` \n\n## What do I do with the output?\n\nThe default output provides two files for user: `output_create.csv` and `output_create.csv`. These files can be used with the Bulk Create and Bulk Delete options in Azure Portal in the Azure AD Users menu. Additionally, one file is created for each defined group with the members of those groups. These files can be used with the Import Members and Remove Members options in Azure Portal in the Azure AD Group menu.\n',
    'author': 'Leron Gray',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/daddycocoaman/AzDummy',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
